<?php
    namespace Suprimentos\Logic\CadProd;

    use \Geral\Entities\Paramgeral;
    use \Geral\Util\Ambiente;
    use \Geral\Util\PictureProduto;
    use \Geral\Util\Novocodigo;
    use \Geral\Entities\Produto;
    use Setforms2\Runtime\Helper\Procedure;
    use Setforms2\Runtime\Helper\Procedure\Param;

    class CadProd{

        const TAM_MAX_CDPRODUTO=10;
        
        /**
         * Busca os parâmetros gerais da empresa que está logada
         * @param string $nrorg
         * @return Paramgeral
         */
        public static function buscaMascara($nrorg){
            
            $where = array($nrorg);
            return (Paramgeral::getObjectByWhere('NRORG = ?',$where));
        }
        /**
         * Retorna a máscara de produtos em forma de array
         * @param string $nrorg
         * @return array string
         */
        public static function buscaNiveis($nrorg){
            
            $mascara = self::buscaMascara($nrorg);            
            return explode('.', $mascara->getCdpictprod());
        }
        /**
         * Retorna quantos dígitos tem a máscara
         * @param string $nrorg
         * @return int
         */
        public static function tamMascara($nrorg){
            $mascara = self::buscaNiveis($nrorg);
            $mascara = implode('',$mascara);
            return (strlen($mascara));
        }
		
		/**
         * Transforma o código inserido em um array parametrizado pela máscara.
         * @param array $mascara
         * @param string $cod
         * @return array
         */
        public static function separaNivel($mascara, $cod){
            $cont = strlen((string)$cod);
            $total = 0;
            $niveis = array();
            
            foreach ($mascara as $masc) {
                $tam = strlen($masc);
                if($cont>$total){
                    $niveis[] = substr($cod, $total, $tam);
                }else
                    break;
                $total+=$tam;
            }

            return($niveis);
        }
		
        
        
        /**
         * Retorna o último código de árvore inserido qeu tenha o código do produto de nível anterior parametrizado 
         * @param string $nrorg
         * @param string $cdprodnivante
         * @return array
         */
        public static function ultCodigo($nrorg, $cdprodnivante){
            
            return (\Geral\Repositories\Produto::ultiArvProd($nrorg, $cdprodnivante));
        }
		
		/**
         * Gerar o próximo código do produto 
         * @param array $mascara
         * @param string $nrorg
         * @param string $cdprodnivante
         * @return int or NULL
         */
        public static function geraProx($mascara, $nrorg, $cdprodnivante){
            
            //Busca o último código
            $ultCod = self::ultCodigo($nrorg,$cdprodnivante);
            if(!$ultCod[0]['ULTCOD']){
                $mascaraGeral = self::buscaNiveis($nrorg);
                $mascara = self::separaNivel($mascara, $cdprodnivante);
                $pos = count($mascara);
                $tam = strlen($mascaraGeral[$pos]);

                $complCod = str_pad('', $tam, 0);
                $cod = $cdprodnivante.$complCod;
                return $cod;
            }
            
            //Constroi um array com os códigos segundo a mascara
            $ultCod = self::separaNivel($mascara, $ultCod[0]['ULTCOD']);
            
            //Define prodriedade de testes
            $n_ultCod = count($ultCod);
            $len_ultCod = strlen($ultCod[$n_ultCod-1]);
            $int_ultCod = (int)$ultCod[$n_ultCod-1];

            //Gera novo código
            $novoCod =(string) ++$int_ultCod;
            $novoCod = self::Zerofill_Before($len_ultCod, $novoCod);
            if(strlen($novoCod) > strlen($ultCod[$n_ultCod-1]))
                return (NULL);
            else{
                $cod = $cdprodnivante.$novoCod;
                return $cod;
            }
        }
		
        /**
         * Valida o código Arvprod
         * @param string $nrorg
         * @param string $cdArvprod
         * @return int
         */
        public static function validaArvprod($nrorg, $cdArvprod){
            
            //Definição de variáveis

            $mascara = self::buscaNiveis($nrorg);
            $cdArvprod = self::separaNivel($mascara, $cdArvprod);
            $n_cdArvprod = count($cdArvprod);
            $c_cdArvprod = '';

            //Verifica a existencia
            for($c = 0; $c < $n_cdArvprod-1;$c++){
                $c_cdArvprod.= $cdArvprod[$c];
                $object = self::buscaProduto($nrorg,$c_cdArvprod);
                if(!$object)
                    return -1;
            }
            $c_cdArvprod.=$cdArvprod;//[$n_cdArvprod-1];
            $object = self::buscaProduto($nrorg, $cdArvprod);
            if($object)
                return 1;
            else
                return 0;
        }
        /**
         * Busca no banco de dados um produto
         * @param string $nrorg
         * @param string $cdArvprod
         * @param string $cdProduto
         * @return Produto
         */
        public static function buscaProduto($nrorg, $cdArvprod=NULL, $cdProduto=NULL){
            
            if(($cdArvprod != NULL)&&($cdProduto!=NULL)){

                //Transformar em string se forem arrays
                if(is_array($cdArvprod))
                    $cdArvprod = implode('',$cdArvprod);
                if(is_array($cdProduto))
                    $cdProduto = implode('',$cdProduto);

                $where = array('0' => $nrorg,'1' => $cdArvprod,'2' => $cdProduto);                
                $object = Produto::getObjectByWhere('NRORG = ? AND cdArvprod = ? AND CDPRODUTO = ?',$where);
                return $object;

            }elseif(($cdArvprod == NULL)&&($cdProduto!=NULL)){

                //Transformar em string se forem arrays
                if(is_array($cdProduto))
                    $cdProduto = implode('',$cdProduto);

                $where = array('0' => $nrorg, '1' => $cdProduto); 
                $object = Produto::getObjectByWhere('NRORG = ? AND CDPRODUTO = ?',$where);
                return $object;

            }elseif (($cdArvprod != NULL)&&($cdProduto==NULL)) {

                //Transformar em string se forem arrays
                if(is_array($cdArvprod))
                    $cdArvprod = implode('',$cdArvprod);
                $where = array('0' => $nrorg, '1' => $cdArvprod); 
                $object = Produto::getObjectByWhere('NRORG = ? AND CDARVPROD = ?',$where);
                return $object;
            }
            return(-1);
        }
        /**
         * Retorna um cdproduto válido
         * @param string $nrorg
         * @param string $cdArvprod
         * @return string
         */
        public static function proxCdproduto($nrorg, $cdArvprod){
            if(strlen($cdArvprod) <= (self::TAM_MAX_CDPRODUTO))
                return $cdArvprod;
            else
                return Novocodigo::geraCodigo($nrorg, 'PRODUTO');
        }

        /**
         *
         * @param $mascara
         * @param $cdArvprod
         * @return string
         */
        public static function completaArvore($cdArvprod){
//            $c = 0;
//            foreach($mascara as $masc){
//                $tam_masc = strlen($masc);
//                $tam_ArvProd = strlen($cdArvprod[$c]);
//                if(($tam_ArvProd>0)&&($tam_ArvProd<$tam_masc)){
//                    $dif = $tam_masc-$tam_ArvProd;
//                    for($cont=0;$cont<$dif;$cont++){
//                        $cdArvprod[$c].=0;
//                    }
//                }
//                $c++;
//            }
            $nrorg = Ambiente::getNrorg();
            $mascara = self::buscaNiveis($nrorg);
            $cdArvprod = self::separaNivel($mascara,$cdArvprod);
            $masc_U = $mascara[count($cdArvprod) - 1];
            $arvprod_U = $cdArvprod[count($cdArvprod) - 1];

            if (strlen($masc_U) > strlen($arvprod_U)) {
//                while (strlen($masc_U) > strlen($arvprod_U))
//                    $arvprod_U .= '0';

                $cdArvprod[count($cdArvprod)-1]=self::Zerofill_Before(strlen($masc_U),$arvprod_U);
                $cdArvprod = implode('',$cdArvprod);

            }else{
                $cdArvprod = implode('',$cdArvprod);
            }

            return $cdArvprod;
        }

        public static function codArvProd_N(){

            $campo = tk_get_field_value('CDARVPROD');
            $mascara = self::buscaNiveis(Ambiente::getNrorg());
            $cdArvprod = self::separaNivel($mascara, $campo);
            $nome='INSERE_AGRUPA';

            if (count($mascara)==count($cdArvprod)) {
//                $params = array(
//                    'P_CDARVPROD' => $campo,
//                    'P_STEDIT' => 'N'
//                );
//                try{
//                    return tk_call_procedure($nome, $params);
//                }
//                catch (Exception $e){
//                    tk_warning($e->getMessage());
//                }
                try{

                    $procedure = new Procedure($nome);
                    $procedure->addParam(
                        new Param (
                            'P_CDARVPROD',
                            Param::PARAM_INPUT,
                            $campo,
                            Param::PARAM_TYPE_STR
                        )
                    );
                    $procedure->addParam(
                        new Param (
                            'P_STEDIT',
                            Param::PARAM_INPUT,
                            'N',
                            Param::PARAM_TYPE_STR
                        )
                    );
                    return $procedure->call();
                }
                catch (Exception $e){
                    tk_warning($e->getMessage());
                }
//            }
            }
        }

        public static function codArvProd_S(){
//            $campo = tk_get_field_value('CDARVPROD');
//            $params = array(
//                'P_CDARVPROD' => $campo,
//                'P_STEDIT' => 'S'
//            );
//            tk_call_procedure('INSERE_AGRUPA',$params);
        }

        /**
         * @param $len_ultCod
         * @param $novoCod
         * @return mixed
         */
        public static function Zerofill_Before($tam, $string)
        {
            while ($tam > strlen($string))
                $string = '0' . $string;
            return $string;
        }

        public static function pegaNivelAnterior($chave){

            $nrorg = Ambiente::getNrorg();
            $mascara = self::buscaNiveis($nrorg);
            $chave = self::separaNivel($mascara,$chave);
            $cdProdniveant = '';
//            for($c=0;$c < count($chave)-1;$c++){
            for($c=0;$c < count($chave);$c++){
                $cdProdniveant.=$chave[$c];
            }
            return $cdProdniveant;

        }

        public static function pegaNivelProduto($cdArvP){

            $nrorg = Ambiente::getNrorg();
            $mascara = self::buscaNiveis($nrorg);
            $nivel = self::separaNivel($mascara,$cdArvP);

            return count($nivel);
        }


        public static function excluiProduto($nrorg, $cdproduto)
        {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var Produto $produto */
            $produto = Produto::getObjectByWhere("NRORG = ? AND CDPRODUTO = ?", array($nrorg, $cdproduto));
            if(is_object($produto)){
                $produto->delete();

                $connection->flush();
                $connection->commit();
            }
        }


        public static function compCodigoNivel($mascara, $cdarvprod){
            $cont = strlen((string)$cdarvprod);
            $total = 0;

            foreach ($mascara as $masc) {
                $tam = strlen($masc);
                if($cont>$total){
                    $valor = substr($cdarvprod, $total, $tam);
                    if (strlen($valor) <> $tam) {
                        return false;
                    }
                    $total+=$tam;
                }
            }

            return true;
        }
    }